package more.crawler.nmc.quartz.job;

import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import more.crawler.nmc.Utils.BeanUtils;
import more.crawler.nmc.Utils.HttpHysplitUtils;
import more.crawler.nmc.Utils.HttpRequest;
import more.crawler.nmc.Utils.SignUtil;
import more.crawler.nmc.configuration.NmcConfig;
import more.crawler.nmc.quartz.model.NmcData;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.JobKey;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.*;

/**
 * @author zuo
 * @date 2022/12/7 16:18
 */

@Slf4j
public class NmcJob implements Job {

    private final NmcConfig nmcConfig = BeanUtils.getBean(NmcConfig.class);

    /**
     * 如果任务还在执行则跳过改条任务执行
     */
    private static final Map<String, Boolean> jobMap = new HashMap<>();

    /**
     * 若参数变量名修改 QuartzJobController中也需对应修改
     */
    private String parameter;

    public void setParameter(String parameter) {
        this.parameter = parameter;
    }

    @SneakyThrows
    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        JobKey jobKey = context.getJobDetail().getKey();
        String key = jobKey.getGroup() + "|" + jobKey.getName();
        List<NmcData> nmcDataList = new ArrayList<>();
        if (Objects.isNull(jobMap.get(key))) {
            jobMap.put(key, true);
            try {
                //这里写逻辑
                // 1.天气形势分析数据下载
                nmcDataList.addAll(patternsAnalysisDownload());
                //2.卫星云图分析数据下载
                nmcDataList.addAll(satellitePictureDownload());
                //3.降水分析数据下载
                nmcDataList.addAll(rainfallAnalysisDownload());
                //4.风场分析数据下载
                nmcDataList.addAll(windAnalysisDownload());
                //5.气温场分析数据下载
                nmcDataList.addAll(temperatureAnalysisDownload());
                //6.能见度分析数据下载
                nmcDataList.addAll(visibilityAnalysisDownload());
                //7.雷达数据下载
                nmcDataList.addAll(lidarAnalysisDownload());

                for (NmcData nmcData : nmcDataList) {
                    String type = "?type=" + nmcData.getType();
                    String category = Objects.isNull(nmcData.getCategory()) ? "" : "&category=" + nmcData.getCategory();
                    String time = "&dataTime=" + DateFormatUtils.format(nmcData.getTime(), "yyyy/MM/dd HH:mm:ss").replaceAll(" ", "T");
//                    String serverUrl = "http://127.0.0.1:31000/api/weather/image/1"+URLEncoder.encode(type + category + time, "UTF-8");
                    String serverUrl = nmcConfig.getSendUrl() + "/api/weather/image/1" + type + category + time;
                    HttpRequest.uploadImage(nmcData.getUrl(), serverUrl, SignUtil.getSign(nmcConfig.getSignKey(), nmcConfig.getSignUserId()));
                }

            } finally {
                jobMap.remove(key);
            }
        }
    }

    // 1.天气形势分析数据下载
    public List<NmcData> patternsAnalysisDownload() {
        String url = "/publish/observations/china/dm/weatherchart-h000.htm";
        String categoryId = ".p-nav.nav2 a";
        Integer type = 1;
        return getNmcData(url, categoryId, type);
    }

    //2.卫星云图分析数据下载
    public List<NmcData> satellitePictureDownload() {
        String url = "/publish/satellite/FY4A-true-color.htm";
        String categoryId = ".p-nav.nav1 a";
        Integer type = 2;
        return getNmcData(url, categoryId, type);
    }

    //3.降水分析数据下载
    public List<NmcData> rainfallAnalysisDownload() {
        String url = "/publish/observations/hourly-precipitation.html";
        String categoryId = ".p-nav.nav1 a";
        Integer type = 3;
        return getNmcData(url, categoryId, type);
    }

    //4.风场分析数据下载
    public List<NmcData> windAnalysisDownload() {
        String url = "/publish/observations/hourly-winds.html";
        Integer type = 4;
        return getNmcData(url, null, type);
    }

    //5.气温场分析数据下载
    public List<NmcData> temperatureAnalysisDownload() {
        String url = "/publish/observations/hourly-temperature.html";
        String categoryId = ".p-nav.nav1 a";
        Integer type = 5;
        return getNmcData(url, categoryId, type);
    }

    //6.能见度分析数据下载
    public List<NmcData> visibilityAnalysisDownload() {
        String url = "/publish/sea/seaplatform1.html";
        Integer type = 6;
        return getNmcData(url, null, type);
    }

    //7.雷达数据下载
    public List<NmcData> lidarAnalysisDownload() {
        String url = "/publish/radar/chinaall.html";
        Integer type = 7;
        return getNmcData(url, null, type);
    }

    private List<NmcData> getNmcData(String url, String categoryId, Integer type) {
        /**
         * 获取要查询的信息
         */
        Map<String, Object> map = HttpHysplitUtils.sendGet(nmcConfig.getHost() + url, null, null, true);
//        Thread.sleep(500);
        String html = map.containsKey("html") ? map.get("html").toString() : "";
        //采用Jsoup解析
        Document doc = Jsoup.parse(html);
        Map<String, String> categoryMap = new LinkedHashMap<>();
        if (StringUtils.isNotBlank(categoryId)) {
            Elements elements = doc.select(categoryId);
            for (Element element : elements) {
                String category = element.text();
                String categoryUrl = element.attr("href");
                categoryMap.put(category, categoryUrl);
            }
        } else {
            categoryMap.put("no_data", url);
        }


        /**
         * 查询信息并保存
         */
        List<NmcData> nmcDataList = new ArrayList<>();
        for (Map.Entry<String, String> entry : categoryMap.entrySet()) {
            String category = entry.getKey();
            String categoryUrl = entry.getValue();
            map = HttpHysplitUtils.sendGet(nmcConfig.getHost() + categoryUrl, null, null, true);
//            Thread.sleep(500);
            html = map.containsKey("html") ? map.get("html").toString() : "";
            doc = Jsoup.parse(html);
            //获取时间
            Elements timeElements = doc.select("#myTabContent .time");
            for (Element element : timeElements) {
                String imgUrl = element.attr("data-img");
                String timeStr = element.attr("data-time");
                String[] split = imgUrl.split("=");
                String year = null;
                if (split.length >= 2) {
                    String timeLong = split[1];
                    Date date = new Date(Long.parseLong(timeLong));
                    year = DateFormatUtils.format(date, "yyyy");
                }

                try {
                    if (StringUtils.isNotEmpty(year)) {
                        timeStr = year + "/" + timeStr;
                        Date time = DateUtils.parseDate(timeStr, "yyyy/MM/dd HH:mm");
                        NmcData nmcData = new NmcData();
                        nmcData.setCategory("no_data".equals(category) ? null : category);
                        nmcData.setType(type);
                        nmcData.setTime(time);
                        nmcData.setUrl(imgUrl);
                        nmcDataList.add(nmcData);
                    }
                } catch (ParseException e) {
                    System.out.println("时间转换错误");
                }
            }
        }
        return nmcDataList;
    }

}
